"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.lte = lte;
exports.gte = gte;
exports.eq = eq;
// predicate should return true if we should go right.
function lte(array, value, cmp) {
    var len = array.length;
    if (len === 0 || !(cmp(array[0], value) <= 0)) {
        return -1;
    }

    var l = 0;
    var r = len;
    while (r - l > 1) {
        // In our case, r + l <= 128 so we don't need
        // to worry about overflow here
        var mid = r + l >>> 1;
        if (cmp(array[mid], value) <= 0) {
            // value at `mid` is less than or equal to `value`, go right.
            l = mid;
        } else {
            // if value at `mid` is larger than `value`, go left.
            r = mid;
        }
    }

    return l;
}

// predicate should return true if we should go left
function gte(array, value, cmp) {
    var len = array.length;
    if (len === 0 || !(cmp(array[len - 1], value) >= 0)) return len;
    var l = -1;
    var r = len - 1;
    while (r - l > 1) {
        var mid = r + l >>> 1;
        if (cmp(array[mid], value) >= 0) {
            // value at `mid` is less than or equal to `value`, go right.
            r = mid;
        } else {
            // if value at `mid` is larger than `value`, go left.
            l = mid;
        }
    }

    return r;
}

function eq(array, value, cmp) {
    var idx = lte(array, value, cmp);
    if (idx !== -1 && cmp(array[idx], value) === 0) {
        return idx;
    }
    return -1;
}

exports.default = {
    lte: lte,
    gte: gte,
    eq: eq
};