'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
var MIN_ROOT_CHILDREN = exports.MIN_ROOT_CHILDREN = 2;

// These numbers are optimized for the Path implementation.
// With these values we can store paths in the binary representation
// of an integer - the lowest 30 bits divide to 5 parts of 6 bits each.
// During range searches and iteration, we check if we've reached the
// end path at each element. When the path is represented as an integer,
// we can do a fast integer comparison.
//
// 6 bits gives us a range of 2^6 = 64 values for each part.
// The maximum tree size with these limitations should be more than enough,
// 64^5 - 1 = 1,073,741,823, that's about 1 billion elements.
var ORDER = exports.ORDER = 64;
var SHIFT_LEN = exports.SHIFT_LEN = 6;

var LEAF_MIN_CHILDREN = exports.LEAF_MIN_CHILDREN = Math.ceil(ORDER / 2) - 1;
var LEAF_MAX_CHILDREN = exports.LEAF_MAX_CHILDREN = ORDER - 1;
var INTERNAL_MIN_CHILDREN = exports.INTERNAL_MIN_CHILDREN = Math.ceil(ORDER / 2);
var INTERNAL_MAX_CHILDREN = exports.INTERNAL_MAX_CHILDREN = ORDER;

var ITERATOR_PROPNAME = exports.ITERATOR_PROPNAME = typeof Symbol === 'function' ? Symbol.iterator : '@@iterator';