'use strict';

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _chai = require('chai');

var _chai2 = _interopRequireDefault(_chai);

var _sinon = require('sinon');

var _sinon2 = _interopRequireDefault(_sinon);

var _sinonChai = require('sinon-chai');

var _sinonChai2 = _interopRequireDefault(_sinonChai);

var _constants = require('../constants');

var _nodes = require('../nodes');

var _index = require('../index');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var cmp = _index.defaultComparator;

_chai2.default.use(_sinonChai2.default);
var expect = _chai2.default.expect;

// Checks that keys and children are shallowequal.

function nodesEqual(a, b) {
    expect(a.keys).to.have.lengthOf(b.keys.length);
    expect(a.children).to.have.lengthOf(b.children.length);

    a.keys.forEach(function (key, i) {
        expect(key).to.equal(b.keys[i]);
    });

    a.children.forEach(function (key, i) {
        expect(key).to.equal(b.children[i]);
    });
}

describe('Node', function () {
    it('constructor works', function () {
        var node = new _nodes.Node();
        expect(node.children).to.deep.equal([]);
        expect(node.keys).to.deep.equal([]);
        expect(node.order).to.deep.equal(_constants.ORDER);
    });

    describe('methods', function () {
        var node = void 0;

        beforeEach(function () {
            node = new _nodes.Node({
                keys: ['1', '2'],
                children: [1, 2, 3]
            });
        });

        it('size', function () {
            expect(node.size).to.equal(3);
        });

        it('satisfiesMinChildren', function () {
            node.minChildren = 2;
            expect(node.satisfiesMinChildren()).to.be.true;
            node.minChildren = 4;
            expect(node.satisfiesMinChildren()).to.be.false;
        });

        it('satisfiesMaxChildren', function () {
            node.maxChildren = 4;
            expect(node.satisfiesMaxChildren()).to.be.true;
            node.maxChildren = 2;
            expect(node.satisfiesMaxChildren()).to.be.false;
        });

        it('tail', function () {
            var tailNode = node.tail();
            expect(tailNode.keys).to.deep.equal(['2']);
            expect(tailNode.children).to.deep.equal([2, 3]);
            expect(tailNode.order).to.deep.equal(node.order);
        });

        it('init', function () {
            var tailNode = node.init();
            expect(tailNode.keys).to.deep.equal(['1']);
            expect(tailNode.children).to.deep.equal([1, 2]);
            expect(tailNode.order).to.deep.equal(node.order);
        });

        it('shouldSplit', function () {
            node.maxChildren = 2;
            expect(node.shouldSplit()).to.be.true;
            node.maxChildren = 4;
            expect(node.shouldSplit()).to.be.false;
        });
    });
});

describe('Leaf', function () {
    it('constructor works', function () {
        var node = new _nodes.Leaf();
        expect(node.children).to.deep.equal([]);
        expect(node.keys).to.deep.equal([]);
        expect(node.order).to.deep.equal(_constants.ORDER);
    });

    describe('methods', function () {
        var leaf = void 0;

        beforeEach(function () {
            leaf = new _nodes.Leaf({
                keys: ['1', '2', '3'],
                children: [1, 2, 3]
            });
        });

        it('delete', function () {
            var leafAfterDeletion = leaf.delete(cmp, '2');

            expect(leafAfterDeletion.keys).to.not.equal(leaf.keys);
            expect(leafAfterDeletion.children).to.not.equal(leaf.children);

            expect(leafAfterDeletion.keys).to.deep.equal(['1', '3']);
            expect(leafAfterDeletion.children).to.deep.equal([1, 3]);
        });

        it('delete when key doesn\'t exist', function () {
            var leafAfterDeletion = leaf.delete(cmp, '4');
            expect(leafAfterDeletion).to.equal(leaf);
            expect(leafAfterDeletion.keys).to.deep.equal(['1', '2', '3']);
            expect(leafAfterDeletion.children).to.deep.equal([1, 2, 3]);
        });

        it('merge', function () {
            var otherLeaf = new _nodes.Leaf({
                keys: ['4', '5'],
                children: [4, 5]
            });
            var merged = leaf.merge(otherLeaf);

            expect(merged.keys).to.deep.equal(['1', '2', '3', '4', '5']);
            expect(merged.children).to.deep.equal([1, 2, 3, 4, 5]);
        });

        it('idxForKey', function () {
            var idx = leaf.idxForKey(cmp, '2');
            expect(idx).to.equal(1);
        });

        it('idxForKey - search key smaller than leaf keys', function () {
            var idx = leaf.idxForKey(cmp, '0');
            expect(idx).to.equal(0);
        });

        it('idxForKey - search key larger than leaf keys', function () {
            var idx = leaf.idxForKey(cmp, '4');
            expect(idx).to.equal(3);
        });

        it('split', function () {
            var otherLeaf = new _nodes.Leaf({
                keys: ['4', '5'],
                children: [4, 5]
            });

            var merged = leaf.merge(otherLeaf);

            var _merged$split = merged.split();

            var _merged$split2 = _slicedToArray(_merged$split, 3);

            var key = _merged$split2[0];
            var firstLeaf = _merged$split2[1];
            var secondLeaf = _merged$split2[2];


            expect(key).to.equal('4');
            expect(firstLeaf.keys).to.deep.equal(['1', '2', '3']);
            expect(firstLeaf.children).to.deep.equal([1, 2, 3]);

            expect(secondLeaf.keys).to.deep.equal(['4', '5']);
            expect(secondLeaf.children).to.deep.equal([4, 5]);
        });

        it('insert', function () {
            var value = 999;
            // '25' should be inserted between '2' and '3'
            // due to lexicographic order.
            var inserted = leaf.insert(cmp, '25', value);

            expect(inserted.keys).to.deep.equal(['1', '2', '25', '3']);
            expect(inserted.children).to.deep.equal([1, 2, value, 3]);
        });

        it('insert smallest key', function () {
            var value = 0;
            var inserted = leaf.insert(cmp, '0', value);

            expect(inserted.keys).to.deep.equal(['0', '1', '2', '3']);
            expect(inserted.children).to.deep.equal([value, 1, 2, 3]);
        });

        it('insert largest key', function () {
            var value = 4;
            var inserted = leaf.insert(cmp, '4', value);

            expect(inserted.keys).to.deep.equal(['1', '2', '3', '4']);
            expect(inserted.children).to.deep.equal([1, 2, 3, value]);
        });

        it('insert - key already exists', function () {
            var value = 999;
            var inserted = leaf.insert(cmp, '2', value);

            expect(inserted).to.not.equal(leaf);
            expect(inserted.keys).to.deep.equal(['1', '2', '3']);
            expect(inserted.children).to.deep.equal([1, value, 3]);
        });

        it('insert - key already exists and has equal value', function () {
            var value = 2;
            var inserted = leaf.insert(cmp, '2', value);
            expect(inserted).to.equal(leaf);
        });

        it('insert - node exceeds maximum children', function () {
            var origShouldSplit = _nodes.Node.prototype.shouldSplit;
            _nodes.Node.prototype.shouldSplit = function () {
                return true;
            };

            var insertedKey = '25';
            var value = 999;

            var _leaf$insert = leaf.insert(cmp, insertedKey, value);

            var _leaf$insert2 = _slicedToArray(_leaf$insert, 3);

            var key = _leaf$insert2[0];
            var firstLeaf = _leaf$insert2[1];
            var secondLeaf = _leaf$insert2[2];


            expect(key).to.equal(insertedKey);
            expect(firstLeaf.keys).to.deep.equal(['1', '2']);
            expect(firstLeaf.children).to.deep.equal([1, 2]);

            expect(secondLeaf.keys).to.deep.equal([insertedKey, '3']);
            expect(secondLeaf.children).to.deep.equal([value, 3]);

            _nodes.Node.prototype.shouldSplit = origShouldSplit;
        });

        describe('rearrangement', function () {
            var left = void 0;
            var right = void 0;

            beforeEach(function () {
                left = new _nodes.Leaf({
                    keys: [1, 2],
                    children: [1, 2]
                });
                right = new _nodes.Leaf({
                    keys: [3, 4],
                    children: [3, 4]
                });
            });

            it('stealFirstKeyFrom', function () {
                var _left$stealFirstKeyFr = left.stealFirstKeyFrom(right);

                var _left$stealFirstKeyFr2 = _slicedToArray(_left$stealFirstKeyFr, 2);

                var newLeft = _left$stealFirstKeyFr2[0];
                var newRight = _left$stealFirstKeyFr2[1];

                // This operation mutates `left`.

                expect(newLeft).to.equal(left);
                expect(newLeft.keys).to.deep.equal([1, 2, 3]);
                expect(newLeft.children).to.deep.equal([1, 2, 3]);

                expect(newRight).not.to.equal(right);
                expect(newRight.keys).to.deep.equal([4]);
                expect(newRight.children).to.deep.equal([4]);
            });

            it('giveLastKeyTo', function () {
                var _left$giveLastKeyTo = left.giveLastKeyTo(right);

                var _left$giveLastKeyTo2 = _slicedToArray(_left$giveLastKeyTo, 2);

                var newLeft = _left$giveLastKeyTo2[0];
                var newRight = _left$giveLastKeyTo2[1];

                // This operation mutates `right`.

                expect(newRight).to.equal(right);
                expect(newRight.keys).to.deep.equal([2, 3, 4]);
                expect(newRight.children).to.deep.equal([2, 3, 4]);

                expect(newLeft).not.to.equal(left);
                expect(newLeft.keys).to.deep.equal([1]);
                expect(newLeft.children).to.deep.equal([1]);
            });
        });
    });
});

describe('InternalNode', function () {
    it('constructor works', function () {
        var node = new _nodes.InternalNode();
        expect(node.children).to.deep.equal([]);
        expect(node.keys).to.deep.equal([]);
        expect(node.order).to.deep.equal(_constants.ORDER);
    });

    describe('methods', function () {
        var node = void 0;

        beforeEach(function () {
            node = new _nodes.InternalNode({
                keys: ['2', '4', '6'],
                children: [{ val: 1 }, { val: 3 }, { val: 5 }, { val: 7 }]
            });
        });

        it('withReplacedChildren', function () {
            var replaced = node.withReplacedChildren(1, ['first', 'second']);
            expect(replaced.keys).to.equal(node.keys);
            expect(replaced.children[0]).to.equal(node.children[0]);
            expect(replaced.children[1]).to.equal('first');
            expect(replaced.children[2]).to.equal('second');
            expect(replaced.children[3]).to.equal(node.children[3]);
        });

        it('smallestKey', function () {
            node.children[0] = new _nodes.InternalNode({
                keys: ['1'],
                children: [new _nodes.Leaf({ keys: ['0', '05'], children: [0, 0.5] }), new _nodes.Leaf({ keys: ['1'], children: [1] })]
            });

            expect(node.smallestKey()).to.equal('0');
        });

        it('childIdxForKey', function () {
            var idx = node.childIdxForKey(cmp, '2');
            expect(idx).to.equal(1);
        });

        it('childIdxForKey - between two keys', function () {
            var idx = node.childIdxForKey(cmp, '3');
            expect(idx).to.equal(1);
        });

        it('childIdxForKey - search key smaller than all keys', function () {
            var idx = node.childIdxForKey(cmp, '1');
            expect(idx).to.equal(0);
        });

        it('childIdxForKey - search key larger than all keys', function () {
            var idx = node.childIdxForKey(cmp, '7');
            expect(idx).to.equal(3);
        });

        it('merge', function () {
            var otherNode = new _nodes.InternalNode({
                keys: ['8'],
                children: [new _nodes.Leaf({ keys: ['7'], children: [7] }), new _nodes.Leaf({ keys: ['8'], children: [8] })]
            });

            var merged = node.merge(otherNode);

            expect(merged.keys).to.deep.equal(['2', '4', '6', '7', '8']);
            expect(merged.children).to.have.lengthOf(6);

            expect(merged.children[0]).to.equal(node.children[0]);
            expect(merged.children[1]).to.equal(node.children[1]);
            expect(merged.children[2]).to.equal(node.children[2]);
            expect(merged.children[3]).to.equal(node.children[3]);
            expect(merged.children[4]).to.equal(otherNode.children[0]);
            expect(merged.children[5]).to.equal(otherNode.children[1]);
        });

        it('split', function () {
            var _node$split = node.split();

            var _node$split2 = _slicedToArray(_node$split, 3);

            var key = _node$split2[0];
            var leftNode = _node$split2[1];
            var rightNode = _node$split2[2];

            expect(key).to.equal('4');

            expect(leftNode.keys).to.deep.equal(['2']);
            expect(leftNode.children).to.have.lengthOf(2);
            expect(leftNode.children[0]).to.equal(node.children[0]);
            expect(leftNode.children[1]).to.equal(node.children[1]);

            expect(rightNode.keys).to.deep.equal(['6']);
            expect(rightNode.children).to.have.lengthOf(2);
            expect(rightNode.children[0]).to.equal(node.children[2]);
            expect(rightNode.children[1]).to.equal(node.children[3]);
        });

        it('withSplitChild', function () {
            var splitKey = '5';
            var origChild = {};
            var splitChild = {};

            var result = node.withSplitChild(cmp, splitKey, origChild, splitChild);

            expect(result.keys).to.deep.equal(['2', '4', '5', '6']);

            expect(result.children).to.have.lengthOf(node.children.length + 1);
            expect(result.children[0]).to.equal(node.children[0]);
            expect(result.children[1]).to.equal(node.children[1]);
            expect(result.children[2]).to.equal(origChild);
            expect(result.children[3]).to.equal(splitChild);
            expect(result.children[4]).to.equal(node.children[3]);
        });

        describe('insert', function () {
            beforeEach(function () {
                node = new _nodes.InternalNode({
                    keys: ['2', '4', '6'],
                    children: [new _nodes.Leaf({
                        keys: ['0', '1'],
                        children: [0, 1]
                    }), new _nodes.Leaf({
                        keys: ['2', '3'],
                        children: [2, 3]
                    }), new _nodes.Leaf({
                        keys: ['4', '5'],
                        children: [4, 5]
                    }), new _nodes.Leaf({
                        keys: ['6', '7'],
                        children: [6, 7]
                    })]
                });
            });

            it('basic case', function () {
                var childInsertSpy = _sinon2.default.spy(node.children[1], 'insert');
                var value = 999;
                var inserted = node.insert(cmp, '3', value);

                expect(childInsertSpy).to.have.been.calledWith(cmp, '3', value);

                expect(inserted.keys).to.equal(node.keys);
                expect(inserted.children).to.have.lengthOf(node.children.length);

                expect(inserted.children[0]).to.equal(node.children[0]);
                expect(inserted.children[1]).not.to.equal(node.children[1]);
                expect(inserted.children[2]).to.equal(node.children[2]);
                expect(inserted.children[3]).to.equal(node.children[3]);
            });

            it('returns the same child (no changes in child)', function () {
                var inserted = node.insert(cmp, '2', 2);
                expect(inserted).to.equal(node);
            });

            it('child is split', function () {
                var origShouldSplit = _nodes.Leaf.prototype.shouldSplit;
                _nodes.Leaf.prototype.shouldSplit = function () {
                    return true;
                };

                var inserted = node.insert(cmp, '25', 2.5);
                expect(inserted.keys).to.have.lengthOf(node.keys.length + 1);
                expect(inserted.keys).to.deep.equal(['2', '3', '4', '6']);

                expect(inserted.children).to.have.lengthOf(node.children.length + 1);
                expect(inserted.children[0]).to.equal(node.children[0]);

                expect(inserted.children[1].keys).to.deep.equal(['2', '25']);
                expect(inserted.children[1].children).to.deep.equal([2, 2.5]);

                expect(inserted.children[2].keys).to.deep.equal(['3']);
                expect(inserted.children[2].children).to.deep.equal([3]);

                expect(inserted.children[3]).to.equal(node.children[2]);
                expect(inserted.children[4]).to.equal(node.children[3]);

                _nodes.Leaf.prototype.shouldSplit = origShouldSplit;
            });
        });

        describe('rearrangement', function () {
            var left = void 0;
            var right = void 0;
            beforeEach(function () {
                left = new _nodes.InternalNode({
                    keys: [2, 4],
                    children: [new _nodes.Leaf({
                        keys: [0, 1],
                        children: [0, 1]
                    }), new _nodes.Leaf({
                        keys: [2, 3],
                        children: [2, 3]
                    }), new _nodes.Leaf({
                        keys: [4],
                        children: [4]
                    })]
                });
                right = new _nodes.InternalNode({
                    keys: [6, 8],
                    children: [new _nodes.Leaf({
                        keys: [5],
                        children: [5]
                    }), new _nodes.Leaf({
                        keys: [6, 7],
                        children: [6, 7]
                    }), new _nodes.Leaf({
                        keys: [8, 9],
                        children: [8, 9]
                    })]
                });
            });

            it('withMergedChildren', function () {
                var mergeSpy = _sinon2.default.spy(left, 'merge');

                var parentNode = new _nodes.InternalNode({
                    keys: [0, 5, 10],
                    children: [{}, left, right, {}]
                });

                // Usually you would supply a new left or right
                // node but for this test it doesn't matter.
                var newParent = parentNode.withMergedChildren(1, left, right);

                expect(mergeSpy).to.have.been.calledOnce;
                var call = mergeSpy.firstCall;
                expect(call.thisValue).to.equal(left);
                expect(call).to.have.been.calledWith(right);
                var mergedChild = call.returnValue;

                expect(newParent.keys).to.deep.equal([0, 10]);
                expect(newParent.children[0]).to.equal(parentNode.children[0]);
                expect(newParent.children[1]).to.equal(mergedChild);
                expect(newParent.children[2]).to.equal(parentNode.children[3]);
            });

            it('withMergedChildren - leftmost nodes', function () {
                var mergeSpy = _sinon2.default.spy(left, 'merge');

                var parentNode = new _nodes.InternalNode({
                    keys: [5, 10],
                    children: [left, right, {}]
                });

                // Usually you would supply a new left or right
                // node but for this test it doesn't matter.
                var newParent = parentNode.withMergedChildren(0, left, right);

                expect(mergeSpy).to.have.been.calledOnce;
                var call = mergeSpy.firstCall;
                expect(call.thisValue).to.equal(left);
                expect(call).to.have.been.calledWith(right);
                var mergedChild = call.returnValue;

                expect(newParent.keys).to.deep.equal([10]);
                expect(newParent.children[0]).to.equal(mergedChild);
                expect(newParent.children[1]).to.equal(parentNode.children[2]);
            });

            it('stealFirstKeyFrom', function () {
                var _left$stealFirstKeyFr3 = left.stealFirstKeyFrom(right);

                var _left$stealFirstKeyFr4 = _slicedToArray(_left$stealFirstKeyFr3, 2);

                var newLeft = _left$stealFirstKeyFr4[0];
                var newRight = _left$stealFirstKeyFr4[1];

                // this operation mutates `left`.

                expect(newLeft).to.equal(left);
                expect(newRight).not.to.equal(right);

                expect(newLeft.keys).to.deep.equal([2, 4, 5]);
                expect(newLeft.children).to.have.lengthOf(4);
                expect(newLeft.children[3]).to.equal(right.children[0]);

                expect(newRight.keys).to.deep.equal([8]);
                expect(newRight.children).to.have.lengthOf(2);
            });

            it('giveLastKeyTo', function () {
                var initialRightChildren = right.children;
                var initialLeftChildren = left.children;

                var _left$giveLastKeyTo3 = left.giveLastKeyTo(right);

                var _left$giveLastKeyTo4 = _slicedToArray(_left$giveLastKeyTo3, 2);

                var newLeft = _left$giveLastKeyTo4[0];
                var newRight = _left$giveLastKeyTo4[1];

                // This operation mutatest `right`.

                expect(newRight).to.equal(right);
                expect(newLeft).to.not.equal(left);

                expect(newLeft.keys).to.deep.equal([2]);
                expect(newLeft.children).to.have.lengthOf(2);
                expect(newLeft.children[0]).to.equal(left.children[0]);
                expect(newLeft.children[1]).to.equal(left.children[1]);

                expect(newRight.keys).to.deep.equal([5, 6, 8]);
                expect(newRight.children).to.have.lengthOf(4);
                expect(newRight.children[0]).to.equal(initialLeftChildren[2]);
                expect(newRight.children[1]).to.equal(initialRightChildren[0]);
                expect(newRight.children[2]).to.equal(initialRightChildren[1]);
                expect(newRight.children[3]).to.equal(initialRightChildren[2]);
            });
        });

        describe('delete', function () {
            beforeEach(function () {
                node = new _nodes.InternalNode({
                    keys: ['2', '4', '6'],
                    children: [new _nodes.Leaf({
                        keys: ['0', '1'],
                        children: [0, 1]
                    }), new _nodes.Leaf({
                        keys: ['2', '3'],
                        children: [2, 3]
                    }), new _nodes.Leaf({
                        keys: ['4', '5'],
                        children: [4, 5]
                    }), new _nodes.Leaf({
                        keys: ['6', '7'],
                        children: [6, 7]
                    })]
                });
            });

            it('normal case', function () {
                // Stop from merging
                var _stub = _sinon2.default.stub(_nodes.Leaf.prototype, 'satisfiesMinChildren');
                _stub.returns(true);

                var newNode = node.delete(cmp, '2');
                expect(newNode.children[0]).to.equal(node.children[0]);
                var newLeaf = newNode.children[1];
                expect(newLeaf.keys).to.deep.equal(['3']);
                expect(newLeaf.children).to.deep.equal([3]);
                expect(newNode.children[2]).to.equal(node.children[2]);
                expect(newNode.children[3]).to.equal(node.children[3]);

                _stub.restore();
            });

            it('key doesn\'t exist', function () {
                var _stub = _sinon2.default.stub(_nodes.Leaf.prototype, 'satisfiesMinChildren');
                _stub.returns(true);

                var newNode = node.delete(cmp, '10');
                expect(newNode).to.equal(node);
                _stub.restore();
            });

            it('merge right leaf', function () {
                // Force merging
                var _stub = _sinon2.default.stub(_nodes.Leaf.prototype, 'satisfiesMinChildren');
                _stub.returns(false);

                // When left and right sibling leafs are of the same size,
                // the right one is selected for merging.
                var newNode = node.delete(cmp, '2');

                expect(newNode.keys).to.deep.equal(['3', '6']);

                var newChildren = newNode.children;
                expect(newChildren).to.have.lengthOf(node.children.length - 1);
                expect(newChildren[0]).to.equal(node.children[0]);
                expect(newChildren[1].keys).to.deep.equal(['3', '4', '5']);
                expect(newChildren[1].children).to.deep.equal([3, 4, 5]);
                expect(newChildren[2]).to.equal(node.children[3]);

                _stub.restore();
            });

            it('merge left leaf', function () {
                // Force merging
                var _stub = _sinon2.default.stub(_nodes.Leaf.prototype, 'satisfiesMinChildren');
                _stub.returns(false);

                // Deletes from rightmost leaf - will merge to left one.
                var newNode = node.delete(cmp, '6');
                expect(newNode.keys).to.deep.equal(['2', '4']);

                expect(newNode.children).to.have.lengthOf(node.children.length - 1);
                expect(newNode.children[0]).to.equal(node.children[0]);
                expect(newNode.children[1]).to.equal(node.children[1]);
                expect(newNode.children[2].keys).to.deep.equal(['4', '5', '7']);
                expect(newNode.children[2].children).to.deep.equal([4, 5, 7]);

                _stub.restore();
            });

            it('steal key from left leaf', function () {
                // 3 was deleted
                var newLeaf = new _nodes.Leaf({
                    keys: ['2'],
                    children: [2]
                });

                newLeaf.satisfiesMinChildren = function () {
                    return false;
                };

                // force stealing from left
                _sinon2.default.stub(node, 'chooseComplexDeletionStrategy').returns({
                    leftNodeIdx: 0,
                    rightNode: newLeaf,
                    leftNode: node.children[0],
                    strategy: _nodes.DELETION_STRATEGIES.STEAL_KEY_FROM_LEFT
                });

                var newNode = node.delete(cmp, '3');
                expect(newNode.keys).to.deep.equal(['1', '4', '6']);
                expect(newNode.children[0].keys).to.deep.equal(['0']);
                expect(newNode.children[1].keys).to.deep.equal(['1', '2']);
            });

            it('steal key from right leaf', function () {
                // 3 was deleted
                var newLeaf = new _nodes.Leaf({
                    keys: ['2'],
                    children: [2]
                });

                newLeaf.satisfiesMinChildren = function () {
                    return false;
                };

                // force stealing from left
                _sinon2.default.stub(node, 'chooseComplexDeletionStrategy').returns({
                    leftNodeIdx: 1,
                    leftNode: newLeaf,
                    rightNode: node.children[2],
                    strategy: _nodes.DELETION_STRATEGIES.STEAL_KEY_FROM_RIGHT
                });

                var newNode = node.delete(cmp, '3');
                expect(newNode.keys).to.deep.equal(['2', '5', '6']);
                expect(newNode.children[1].keys).to.deep.equal(['2', '4']);
                expect(newNode.children[2].keys).to.deep.equal(['5']);
            });
        });
    });
});