'use strict';

var _chai = require('chai');

var _chai2 = _interopRequireDefault(_chai);

var _sinonChai = require('sinon-chai');

var _sinonChai2 = _interopRequireDefault(_sinonChai);

var _path2 = require('../path');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_chai2.default.use(_sinonChai2.default);
var expect = _chai2.default.expect;


describe('Path', function () {
    var path = void 0;
    var pathWithValues = void 0;
    beforeEach(function () {
        path = new _path2.Path();

        var pathValue = parseInt('000000' + '000111' + '000000', 2);
        pathWithValues = new _path2.Path(6, 5, pathValue);
    });
    it('constructor works', function () {
        expect(path.shiftLen).to.equal(6);
        expect(path._path).to.equal(0);
    });

    it('from works', function () {
        var pathArr = [1, 2, 3, 4];
        var _path = _path2.Path.from(pathArr);
        expect(_path.get(0)).to.equal(1);
        expect(_path.get(1)).to.equal(2);
        expect(_path.get(2)).to.equal(3);
        expect(_path.get(3)).to.equal(4);
        expect(_path.length).to.equal(pathArr.length);
    });

    it('toArray works', function () {
        var pathArr = [1, 2, 3, 4];
        var _path = _path2.Path.from(pathArr);
        expect(_path.toArray()).to.deep.equal([1, 2, 3, 4]);
    });

    it('setting a value works', function () {
        var newPath = path.set(0, 5);
        expect(newPath._path).to.not.equal(path._path);
        expect(newPath.get(0)).to.equal(5);

        var overwrittenPath = pathWithValues.set(1, 50);
        expect(overwrittenPath.get(1)).to.equal(50);
    });

    it('gets a value', function () {
        expect(path.get(0)).to.equal(0);
        expect(pathWithValues.get(1)).to.equal(7);
    });
});

describe('Bit operations', function () {
    it('bitSlice', function () {
        var num = parseInt('0011111100', 2);
        var expected = parseInt('111111', 2);
        expect((0, _path2.bitSlice)(2, 8, num)).to.equal(expected);
    });

    it('pathGet', function () {
        var lvlShift = 6;
        var firstBin = '010101';
        var first = parseInt(firstBin, 2);

        var secondBin = '010100';
        var second = parseInt(secondBin, 2);

        var thirdBin = '010111';
        var third = parseInt(thirdBin, 2);

        var num = parseInt(thirdBin + secondBin + firstBin, 2);

        expect((0, _path2.pathGet)(lvlShift, 0, num)).to.equal(first);
        expect((0, _path2.pathGet)(lvlShift, 1, num)).to.equal(second);
        expect((0, _path2.pathGet)(lvlShift, 2, num)).to.equal(third);
    });

    it('clearBitRange', function () {
        var bin = '1111';
        var cleared = (0, _path2.clearBitRange)(1, 3, parseInt(bin, 2));
        expect(cleared).to.equal(parseInt('1001', 2));
    });

    it('pathSet', function () {
        var lvlShift = 6;
        var numBin = '000000' + '010101' + '000000';
        var num = parseInt(numBin, 2);

        var result = (0, _path2.pathSet)(lvlShift, 0, num, parseInt('111111', 2));

        expect(result).to.equal(parseInt('000000' + '010101' + '111111', 2));

        var result2 = (0, _path2.pathSet)(lvlShift, 2, num, parseInt('111111', 2));

        expect(result2).to.equal(parseInt('111111' + '010101' + '000000', 2));
    });

    it('safePathSet', function () {
        var lvlShift = 6;
        var numBin = '000000' + '010101' + '111111';
        var num = parseInt(numBin, 2);

        var result = (0, _path2.safePathSet)(lvlShift, 1, num, parseInt('111000', 2));
        expect(result).to.equal(parseInt('000000' + '111000' + '111111', 2));
    });
});