'use strict';

var _chai = require('chai');

var _chai2 = _interopRequireDefault(_chai);

var _sinonChai = require('sinon-chai');

var _sinonChai2 = _interopRequireDefault(_sinonChai);

var _flatten = require('ramda/src/flatten');

var _flatten2 = _interopRequireDefault(_flatten);

var _utils = require('../utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_chai2.default.use(_sinonChai2.default);
var expect = _chai2.default.expect;


describe('Utils', function () {
    it('bounded chunk', function () {
        var size = 102;
        var data = new Array(size);
        var min = 5;
        var max = 10;
        var chunks = (0, _utils.boundedChunk)(min, max, data);
        chunks.forEach(function (chunk) {
            expect(chunk.length).to.be.at.least(min);
            expect(chunk.length).to.be.at.most(max);
        });
        expect((0, _flatten2.default)(chunks).length).to.equal(size);
    });

    it('fastArraySlice', function () {
        var arr = [0, 1, 2, 3, 4];
        var result = (0, _utils.fastArraySlice)(1, 3, arr);
        expect(result).to.deep.equal([1, 2]);
    });

    it('fastMap', function () {
        var arr = [0, 1, 2, 3];
        var result = (0, _utils.fastMap)(function (num) {
            return num.toString();
        }, arr);
        expect(result).to.deep.equal(['0', '1', '2', '3']);
    });

    it('fastInsert', function () {
        var arr = [0, 1, 3, 4];
        var result = (0, _utils.fastInsert)(2, 2, arr);
        expect(result).to.deep.equal([0, 1, 2, 3, 4]);
    });

    it('withoutIdx', function () {
        var arr = [0, 1, 1, 2, 3];
        var result = (0, _utils.withoutIdx)(2, arr);
        expect(result).to.deep.equal([0, 1, 2, 3]);
    });

    it('splitAt', function () {
        var arr = [0, 1, 2, 3];
        var result = (0, _utils.splitAt)(2, arr);
        expect(result).to.deep.equal([[0, 1], [2, 3]]);
    });

    it('unshift', function () {
        var arr = [1, 2, 3];
        var result = (0, _utils.unshift)(0, arr);
        expect(result).to.deep.equal([0, 1, 2, 3]);
    });

    it('takeIdxAndSplit', function () {
        var arr = [1, 2, 3, 4, 5];
        var result = (0, _utils.takeIdxAndSplit)(2, arr);
        expect(result).to.deep.equal([[1, 2], 3, [4, 5]]);
    });
});