'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.boundedChunk = exports.takeIdxAndSplit = exports.median = undefined;
exports.fastArraySlice = fastArraySlice;
exports.fastArrayClone = fastArrayClone;
exports.withoutIdx = withoutIdx;
exports.fastInsert = fastInsert;
exports.fastSet = fastSet;
exports.fastMap = fastMap;
exports.splitAt = splitAt;
exports.unshift = unshift;
exports.last = last;
exports.init = init;
exports.tail = tail;
exports.getEmptyIterator = getEmptyIterator;
exports.extend = extend;

var _constants = require('./constants');

var median = exports.median = function median(len) {
    return Math.ceil(len / 2);
};

function fastArraySlice(start, end, arr) {
    var newArr = new Array(end - start);
    for (var i = start; i < end; i++) {
        newArr[i - start] = arr[i];
    }
    return newArr;
}

function fastArrayClone(arr) {
    var len = arr.length;
    var copy = new Array(len);

    for (var i = 0; i < len; i++) {
        copy[i] = arr[i];
    }
    return copy;
}

function withoutIdx(idx, arr) {
    var copied = fastArrayClone(arr);
    copied.splice(idx, 1);
    return copied;
}

function fastInsert(idx, val, arr) {
    var newArrLen = arr.length + 1;
    var newArr = new Array(newArrLen);

    var i = 0;
    for (; i < idx; i++) {
        newArr[i] = arr[i];
    }

    newArr[i++] = val;

    for (; i < newArrLen; i++) {
        newArr[i] = arr[i - 1];
    }

    return newArr;
}

function fastSet(idx, val, arr) {
    var copied = fastArrayClone(arr);
    copied[idx] = val;
    return copied;
}

function fastMap(fn, arr) {
    var copied = fastArrayClone(arr);
    var len = arr.length;
    for (var i = 0; i < len; i++) {
        copied[i] = fn(arr[i], i);
    }
    return copied;
}

function splitAt(idx, arr) {
    var arrLen = arr.length;
    var firstArrLen = idx;
    var firstArr = new Array(firstArrLen);
    var secondArrLen = arrLen - idx;
    var secondArr = new Array(secondArrLen);

    for (var i = 0; i < idx; i++) {
        firstArr[i] = arr[i];
    }

    for (var _i = idx; _i < arrLen; _i++) {
        secondArr[_i - idx] = arr[_i];
    }

    return [firstArr, secondArr];
}

function unshift(value, arr) {
    return fastInsert(0, value, arr);
}

var takeIdxAndSplit = exports.takeIdxAndSplit = function takeIdxAndSplit(idx, arr) {
    var cutoff = idx;
    var a1len = cutoff;
    var arr1 = fastArraySlice(0, a1len, arr);
    var arr2 = fastArraySlice(cutoff + 1, arr.length, arr);
    return [arr1, arr[cutoff], arr2];
};

function last(arr) {
    return arr[arr.length - 1];
}

function init(arr) {
    if (arr.length <= 1) return [];
    return fastArraySlice(0, arr.length - 1, arr);
}

function tail(arr) {
    if (arr.length <= 1) return [];
    return fastArraySlice(1, arr.length, arr);
}

function getEmptyIterator() {
    var iterator = {
        next: function next() {
            return { done: true };
        }
    };

    iterator[_constants.ITERATOR_PROPNAME] = function () {
        return iterator;
    };

    return iterator;
}

var boundedChunk = exports.boundedChunk = function boundedChunk(min, max, arr) {
    var arrLen = arr.length;
    if (!arr.length) return [];
    if (arr.length <= max) {
        return [arr];
    }

    var avg = Math.ceil((min + max) / 2);
    var parts = arrLen / avg;
    var chunkCount = Math.ceil(parts);
    var splitsize = 1 / chunkCount * arrLen;
    var chunks = new Array(chunkCount);
    for (var i = 0; i < chunkCount; i++) {
        chunks[i] = arr.slice(Math.ceil(i * splitsize), Math.ceil((i + 1) * splitsize));
    }
    return chunks;
};

function extend(target) {
    var argsLen = arguments.length;
    var source = void 0;
    var keys = void 0;
    var key = void 0;
    var i = void 0;
    var j = void 0;

    for (j = 1; j < argsLen; j++) {
        source = arguments[j];
        keys = Object.keys(source);
        for (i = 0; i < keys.length; i++) {
            key = keys[i];
            target[key] = source[key];
        }
    }
    return target;
}